@@ -84,6 +84,7 @@ module Options.Applicative.Builder (
   noBacktrack,
   columns,
   prefs,
+  defaultPrefs,
 
   -- * Types
   Mod,
@@ -135,6 +136,11 @@ long :: HasName f => String -> Mod f a
 long = fieldMod . name . OptLong
 
 -- | Specify a default value for an option.
+--
+-- /Note/: Because this modifier means the parser will never fail,
+-- do not use it with combinators such as 'some' or 'many', as
+-- these combinators continue until a failure occurs.
+-- Careless use will thus result in a hang.
 value :: HasValue f => a -> Mod f a
 value x = Mod id (DefaultProp (Just x) Nothing) id
 
@@ -186,7 +192,7 @@ completeWith xs = completer (listCompleter xs)
 
 -- | Add a bash completion action. Common actions include @file@ and
 -- @directory@. See
--- http://www.gnu.org/software/bash/manual/html_node/Programmable-Completion-Builtins.html#Programmable-Completion-Builtins
+-- <http://www.gnu.org/software/bash/manual/html_node/Programmable-Completion-Builtins.html#Programmable-Completion-Builtins>
 -- for a complete list.
 action :: HasCompleter f => String -> Mod f a
 action act = completer (bashCompleter act)
@@ -388,3 +394,7 @@ prefs m = applyPrefsMod m base
 -- | Trivial option modifier.
 idm :: Monoid m => m
 idm = mempty
+
+-- | Default preferences.
+defaultPrefs :: ParserPrefs
+defaultPrefs = prefs idm
