@@ -58,7 +58,7 @@ hsubparser m = mkParser d g rdr
 -- Parse command line arguments. Display help text and exit if any parse error
 -- occurs.
 execParser :: ParserInfo a -> IO a
-execParser = customExecParser (prefs idm)
+execParser = customExecParser defaultPrefs
 
 -- | Run a program description with custom preferences.
 customExecParser :: ParserPrefs -> ParserInfo a -> IO a
@@ -102,7 +102,7 @@ getParseResult _ = Nothing
 -- If you need to keep track of error messages, use 'execParserPure' instead.
 {-# DEPRECATED execParserMaybe "Use execParserPure together with getParseResult instead" #-}
 execParserMaybe :: ParserInfo a -> [String] -> Maybe a
-execParserMaybe = customExecParserMaybe (prefs idm)
+execParserMaybe = customExecParserMaybe defaultPrefs
 
 -- | Run a program description with custom preferences in pure code.
 --
@@ -143,9 +143,11 @@ parserFailure pprefs pinfo msg ctx = ParserFailure $ \progn ->
   in (h, exit_code, prefColumns pprefs)
   where
     exit_code = case msg of
-      ErrorMsg _   -> ExitFailure (infoFailureCode pinfo)
-      UnknownError -> ExitFailure (infoFailureCode pinfo)
-      _            -> ExitSuccess
+      ErrorMsg _     -> ExitFailure (infoFailureCode pinfo)
+      UnknownError   -> ExitFailure (infoFailureCode pinfo)
+      MissingError _ -> ExitFailure (infoFailureCode pinfo)
+      ShowHelpText   -> ExitSuccess
+      InfoMsg  _     -> ExitSuccess
 
     with_context :: Context
                  -> ParserInfo a
@@ -161,10 +163,11 @@ parserFailure pprefs pinfo msg ctx = ParserFailure $ \progn ->
         , fmap (indent 2) . infoProgDesc $ i ]
 
     error_help = errorHelp $ case msg of
-      ShowHelpText -> mempty
-      ErrorMsg m   -> stringChunk m
-      InfoMsg  m   -> stringChunk m
-      UnknownError -> mempty
+      ShowHelpText   -> mempty
+      ErrorMsg m     -> stringChunk m
+      InfoMsg  m     -> stringChunk m
+      MissingError x -> stringChunk "Missing:" <<+>> fold_tree x
+      UnknownError   -> mempty
 
     base_help :: ParserInfo a -> ParserHelp
     base_help i
